/*
 * $Id$
 *
 * Serial Wire Debug Open API.
 * Library Body File.
 *
 * Copyright (C) 2010, Tomasz Boleslaw CEDRO (http://www.tomek.cedro.info)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the Tomasz Boleslaw CEDRO nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.*
 *
 * Written by Tomasz Boleslaw CEDRO <tomek.cedro@gmail.com>, 2010;
 *
 */

#include <libswd.h>
#include <string.h>
#include <stdlib.h>

/* DATA GENERATED BELOW ARE TO BE SHIFTED OUT MSB FIRST, UNLIKE IN JTAG!
 * These functions generate a bistream packets for SWD communications.
 * Bistream is returned in byte array pointed by *buffer parameter.
 * Actual positive byte count is returned by the function.
 * Negative return value means an error. Zero means no data was generated.
 * If you need to swap bit direction, use bitswap functions provided below.
 * All functions name start with "swd_bitgen_" prefix.
 */

/* Even parity calculator helper function for int type. */
int swd_bin8_parity_even(char data){
 char i,parity=0;
 unsigned char test=data;
 for (i=0;i<=8;i++) parity ^= ((test>>i)&1);
 if (parity<0 || parity>1) return SWD_ERROR_PARITY;
 return parity;
}

int swd_bin32_parity_even(int *data){
 int i,parity=0;
 unsigned int test=*data;
 for (i=0;i<32;i++) parity ^= (test>>i)&1; 
 if (parity<0 || parity>1) return SWD_ERROR_PARITY;
 return parity;
}

int swd_bin8_print(char *data){
 unsigned char i, bits=*data;
 for (i=0;i<8;i++) putchar(((bits<<i)&0x80)?'1':'0'); 
 return i;
}

int swd_bin32_print(int *data){
 unsigned int i, bits=*data;
 for (i=0;i<32;i++) putchar(((bits<<i)&0x80000000)?'1':'0');
 return i;
}

char *swd_bin8_string(char *data){
 static char string[9]; string[8]=0;
 unsigned char i, bits=*data;
 for (i=0;i<8;i++) string[7-i]=(bits&(1<<i))?'1':'0'; 
 return string;
}

char *swd_bin32_string(int *data){
 static char string[33]; string[32]=0;
 unsigned int i, bits=*data;
 for (i=0;i<32;i++) string[31-i]=(bits&(1<<i))?'1':'0';
 return string;
}



/* BitSwap helper function that reverse bit order in *buffer. 
 * Least Significant Bit becomes Most Significant Bit.
 * Rersult contains only "count" bits for use with char or int easily.
 * This function operates on unsigned char (8-bit) data type.
 */
int swd_bin8_bitswap(unsigned char *buffer, int bitcount){
 if (buffer==NULL) return SWD_ERROR_NULLPOINTER;
 if (bitcount>8) return SWD_ERROR_PARAM;
 unsigned char bit, result=0; //res must be unsigned for proper shifting result
 #ifdef __SWDDEBUG__
 printf("|SWD_DEBUG: swd_bin8_bitswap(%02X, %d);\n", *buffer, bitcount);
 #endif
 for (bit=0;bit<bitcount;bit++) {
  result=(result<<1)|(((*buffer>>bit)&1)?1:0); 
  #ifdef __SWDDEBUG__
  printf("|SWD_DEBUG: swd_bin8_bitswap: in=%02X out=%02X bit=%d\n", *buffer, result, bit);
  #endif
 }
 *buffer=result;
 return SWD_OK;
}

/* BitSwap helper function that reverse bit order in *buffer. 
 * Least Significant Bit becomes Most Significant Bit.
 * Rersult contains only "count" bits for use with char or int easily.
 * This function operates on unsigned int (32-bit) data type.
 */
int swd_bin32_bitswap(unsigned int *buffer, int bitcount){
 if (buffer==NULL) return SWD_ERROR_NULLPOINTER;
 if (bitcount>32) return SWD_ERROR_PARAM;
 unsigned int bit, result=0; //res must be unsigned for proper shifting result
 #ifdef __SWDDEBUG__
 printf("|SWD_DEBUG: swd_bin32_bitswap(%08X, %d);\n", *buffer, bitcount);
 #endif
 for (bit=0;bit<bitcount;bit++) {
  result=(result<<1)|(((*buffer>>bit)&1)?1:0); 
  #ifdef __SWDDEBUG__
  printf("|SWD_DEBUG: swd_bin32_bitswap: in=%08X out=%08X bit=%d\n", *buffer, result, bit);
  #endif
 }
 *buffer=result;
 return SWD_OK;
}




/* Command Queue helper functions.
 * All functions start with "swd_cmd_queue_" prefix. 
 */

/* Find first element of a queue pointed by *cmdq element. */
swd_cmd_t* swd_cmd_queue_find_root(swd_cmd_t *cmdq){
 if (cmdq==NULL) return NULL;
 swd_cmd_t *cmd=cmdq;
 while (cmd->prev!=NULL) cmd=cmd->prev;
 return cmd;
}

/* Find last element of a queue pointed by *cmdq element. */
swd_cmd_t* swd_cmd_queue_find_end(swd_cmd_t *cmdq){
 if (cmdq==NULL) return NULL;
 swd_cmd_t *cmd=cmdq;
 while (cmd->next!=NULL) cmd=cmd->next;
 return cmd;
}

/* Append element pointed by *cmd at the end of the quque pointed by *cmdq. */
int swd_cmd_queue_append(swd_cmd_t *cmdq, swd_cmd_t *cmd){
 if (cmdq==NULL) return SWD_ERROR_NULLQUEUE;
 if (cmd==NULL) return SWD_ERROR_NULLPOINTER;
 if (cmdq->next != NULL){
  swd_cmd_t *lastcmd;
  lastcmd=swd_cmd_queue_find_end(cmdq);
  lastcmd->next=cmd;
  cmd->prev=lastcmd;
 } else {
  cmdq->next=cmd;
  cmd->prev=cmdq;
 }
 cmdq=cmd; 
 return 1;
}

/* Free memory / destroy queue pointed by the *cmdq element. */
int swd_cmd_queue_free(swd_cmd_t *cmdq){
 if (cmdq==NULL) return SWD_ERROR_NULLQUEUE;
 int cmdcnt=0;
 swd_cmd_t *cmd, *nextcmd;
 cmd=swd_cmd_queue_find_root(cmdq);
 while (cmd!=NULL) {
  nextcmd=cmd->next;
  free(cmd);
  cmd=nextcmd;
  cmdcnt++;
 }
 return cmdcnt;
}

/* Free memory / destroy / shorten queue head from root to the *cmdq element. */
int swd_cmd_queue_free_head(swd_cmd_t *cmdq){
 if (cmdq==NULL) return SWD_ERROR_NULLQUEUE;
 int cmdcnt=0;
 swd_cmd_t *cmdqroot, *nextcmd;
 cmdqroot=swd_cmd_queue_find_root(cmdq);
 while(cmdqroot!=cmdq){
  nextcmd=cmdqroot->next;
  free(cmdqroot);
  cmdqroot=nextcmd;
  cmdcnt++;
 }
 cmdqroot->prev=NULL;
 return cmdcnt;
}

int swd_cmd_queue_free_tail(swd_cmd_t *cmdq){
 if (cmdq==NULL) return SWD_ERROR_NULLQUEUE;
 int cmdcnt=0;
 swd_cmd_t *cmdqend, *nextcmd;
 nextcmd=cmdq;
 cmdqend=swd_cmd_queue_find_end(cmdq);
 if (cmdqend==NULL) return SWD_ERROR_QUEUE; 
 while(nextcmd!=cmdq){
  nextcmd=cmdqend->prev;
  free(cmdqend);
  cmdqend=nextcmd;
  cmdcnt++;
 }
 cmdqend->next=NULL;
 return cmdcnt;
}





/*******************************************************************************
 * SWD Commands Queue Elements Generator Routines.
 * These command quants are created in memory and can be easily appended
 * to the end of queue pointed by *cmdq element.
 * All functions here start with "swd_cmd_queue_append_" prefix.
 ******************************************************************************/

/* Append SWD Request Packet Header to the context command queue.
 * Note that contents is not validated, so bad request can be sent as well.*/
int swd_cmd_queue_append_request(swd_ctx_t *swdctx, char *request){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 if (request==NULL) return SWD_ERROR_NULLPOINTER;
 swd_cmd_t *cmd;
 cmd=calloc(1,sizeof(swd_cmd_t));
 if (cmd==NULL) return SWD_ERROR_OUTOFMEM;
 cmd->request=*request;
 cmd->bits=SWD_REQ_BITLEN;
 cmd->cmdtype=SWD_CMDTYPE_MOSI_REQUEST;
 return swd_cmd_queue_append(swdctx->cmdq, cmd);
} 

/* Append Turnaround activating MOSI to the context command queue. */ 
int swd_cmd_queue_append_mosi_trn(swd_ctx_t *swdctx){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
  swd_cmd_t *cmd;
  cmd=calloc(1,sizeof(swd_cmd_t));
  if (cmd==NULL) return SWD_ERROR_OUTOFMEM;
  cmd->TRNnMOSI=0;
  cmd->bits=swdctx->params.trnlen;
  cmd->cmdtype=SWD_CMDTYPE_MOSI_TRN;
  return swd_cmd_queue_append(swdctx->cmdq, cmd);
}

/* Append Turnaround activating MISO to the context command queue. */
int swd_cmd_queue_append_miso_trn(swd_ctx_t *swdctx){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
  swd_cmd_t *cmd;
  cmd=calloc(1,sizeof(swd_cmd_t));
  if (cmd==NULL) return SWD_ERROR_OUTOFMEM;
  cmd->TRNnMOSI=1;
  cmd->bits=swdctx->params.trnlen;
  cmd->cmdtype=SWD_CMDTYPE_MISO_TRN;
  return swd_cmd_queue_append(swdctx->cmdq, cmd);
}



/* Append bus binary read bit-by-bit operation to the context command queue.
 * Note that each command will store one bit value into one integer data
 * element, so read is not constrained to 32 bits.
 * On error memory is released and apropriate error code is returned.
 * Important: Memory pointed by *data must be allocated prior call! */
int swd_cmd_queue_append_miso_nbit(swd_ctx_t *swdctx, int **data, int count){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 if (data==NULL) return SWD_ERROR_NULLPOINTER;
 if (count<=0) return SWD_ERROR_PARAM;
 swd_cmd_t **cmd, *tmpcmdq;
 cmd=calloc(count,sizeof(swd_cmd_t));
 if (cmd==NULL) return SWD_ERROR_OUTOFMEM;
 int i;
 for (i=0;i<count;i++){
  data[i]=&cmd[i]->misodata;
  cmd[i]->bits=1;
  cmd[i]->cmdtype=SWD_CMDTYPE_MISO_BITBANG;
  if (swd_cmd_queue_append(swdctx->cmdq, cmd[i])!=1){
   swd_cmd_queue_free_tail(cmd[0]); // if that fails, free memory, return error
   for (i=0;i<count;i++) free(cmd[i]);
   return SWD_ERROR_QUEUE;
  }
 }
 return i;
}

int swd_cmd_queue_append_mosi_nbit(swd_ctx_t *swdctx, int *data, int count){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 if (data==NULL) return SWD_ERROR_NULLPOINTER;
 if (count<=0) return SWD_ERROR_PARAM;
 swd_cmd_t **cmd;
 cmd=calloc(count, sizeof(swd_cmt_t));
 if (cmd==NULL) return SWD_ERROR_OUTOFMEM;
 int i;
 for (i=0;i<count;i++){
  cmd[i]->mosidata=data[i];
  cmd[i]->bits=1;
	 cmd[i]->cmdtype=SWD_CMDTYPE_MOSI_BITBANG;
  if (swd_cmd_queue_append(swdctx->cmdq, cmd[i])!=1){
   swd_cmd_queue_free_tail(cmd[0]); // if that fails, clean memory, return error
   for (i=0;i<count;i++) free(cmd[i]);
   return SWD_ERROR_QUEUE;
  }
 }
 return count;
}

int swd_cmd_queue_append_mosi_parity(swd_ctx_t *swdctx, char *parity){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 if (parity!=0 || parity!=1) return SWD_ERROR_PARAM;
 swd_cmd_t *cmd;
 cmd=calloc(1,sizeof(swd_cmd_t));
 if (swd==NULL) return SWD_ERROR_OUTOFMEM;
 cmd->parity=*parity;
 cmd->bits=1;
 cmd->cmdtype=SWD_CMDTYPE_MOSI_PARITY;
 return swd_cmd_queue_append(swdctx->cmdq, cmd);
}

int swd_cmd_queue_append_miso_parity(swd_ctx_t *swdctx, char *parity){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 if (parity==NULL) return SWD_ERROR_NULLPOINTER;
 swd_cmd_t *cmd;
 cmd=calloc(1,sizeof(swd_cmd_t));
 if (swd==NULL) return SWD_ERROR_OUTOFMEM;
 parity=&cmd->parity;
 cmd->bits=1;
 cmd->cmdtype=SWD_CMDTYPE_MISO_PARITY;
 return swd_cmd_queue_append(swdctx->cmdq, cmd);
}




/* Append 32-bit element + 1-bit parity read operation into context cmd queue.*/
int swd_cmd_queue_append_miso_data(swd_ctx_t *swdctx, int *data){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 if (data==NULL) return SWD_ERROR_NULLPOINTER;
 swd_cmd_t *cmd;
 cmd=calloc(1,sizeof(swd_cmd_t));
 if (cmd==NULL) return SWD_ERROR_OUTOFMEM;
 data=&cmd->misodata;
 cmd->bits=32;
 cmd->cmdtype=SWD_CMDTYPE_MISO_DATA;
 return swd_cmd_queue_append(swdctx->cmdq, cmd); // should be 1 on success
}

/* Append 32-bit element + 1-bit parity read operation into context cmd queue.*/
int swd_cmd_queue_append_miso_data_p(swd_ctx_t *swdctx, int *data, char *parity){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 if (data==NULL) return SWD_ERROR_NULLPOINTER;
 if (parity==NULL) return SWD_ERROR_NULLPOINTER;
 int res, cmdcnt=0;
 res=swd_cmd_queue_append_miso_data(swdctx, data);
 if (ret<0) return res;
 cmdcnt=+res;
 res=swd_cmd_queue_append_miso_parity(swdctx, parity);
 if (ret<0) return res;
 cmdcnt=+res;
 return cmdcnt; // should be 2 on success
}


/* Append n-element read operation into context command queue. */
int swd_cmd_append_miso_n_data_p(swd_ctx_t *swdctx, int **data, char **parity, int count){
 if (swdctx==NULL) ret SWD_ERROR_NULLCONTEXT;
 if (data==NULL) return SWD_ERROR_NULLPOINTER;
 if (count<=0) return SWD_ERROR_PARAM;
 int i,res, cmdcnt=0;
 for (i=0;i<=count;i++){
  res=swd_cmd_queue_append_miso_data_p(swdctx, data[i], parity[i]);
  if (res<2) return SWD_ERROR_RESULT;
  cmdcnt=+res;
 }
 return cmdcnt;
}


int swd_cmd_append_mosi_data(swd_ctx_t *swdctx, int *data){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 if (data==NULL) return SWD_ERROR_NULLPOINTER;
 swd_cmd_t *cmd;
 int ret;
 cmd=calloc(1,sizeof(swd_cmd_t));
 if (cmd==NULL) return SWD_ERROR_OUTOFMEM;
 cmd->mosidata=*data;
 cmd->bits=32;
 cmd->cmdtype=SWD_CMDTYPE_MOSI_DATA;
 return swd_cmd_queue_append(swdctx->cmdq, cmd); // should be 1 on success
}

/* This function adds data and parity into context command queue.
 * Note: This function automatically calculates parity. */
int swd_cmd_queue_append_mosi_data_ap(swd_ctx_t *swdctx, int *data){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 if (data==NULL) return SWD_ERROR_NULLPOINTER
 int res, cmdcnt=0, par;
 res=swd_cmd_queue_append_mosi_data(swdctx, data);
 if (res<1) return res;
 cmdcnt=+ret;
 res=swd_cmd_queue_append_mosi_parity(swdctx, swd_bit32_parity_even(data));
 if (res<1) return res;
 cmdcnt=+res;
 return cmdcnt; // should be 2 on success
}


/* This function will add specified data and parity into context command queue.
 * Note: Parity is not verified, so erratic parity can be supplied. */
int swd_cmd_queue_append_mosi_data_p(swd_ctx_t *swdctx, int *data, char *parity){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 if (data==NULL || parity==NULL) return SWD_ERROR_NULLPOINTER
 int res, cmdcnt=0;
 ret=swd_cmd_queue_append_mosi_data(swdctx, data);
 if (res<1) return ret;
 cmdcnt=+ret;
 ret=swd_cmd_queue_append_mosi_parity(swdctx, parity);
 if (res<1) return ret;
 cmdcnt=+res;
 return cmdcnt; // should be 2 on success
}

int swd_cmd_append_mosi_n_data_ap(swd_ctx_t *swdctx, int **data, int count){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 if (data==NULL) return SWD_ERROR_NULLPOINTER;
 if (count<1) return SWD_ERROR_PARAM;
 int i, res, cmdcnt=0;
 for (i=0;i<count;i++){
  res=swd_cmd_queue_append_mosi_data(swdctx, data[i]);
  if (res<1) return res;
  cmdcnd=+res;
 }
 return cmdcnt;
}

/* Append Acnkowledge request from slave to the command queue. */
int swd_cmd_queue_append_ack(swd_ctx_t *swdctx, char *ack){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 if (ack==NULL) return SWD_ERROR_NULLPOINTER;
 swd_cmd_t *cmd;
 cmd=calloc(1,sizeof(swd_cmd_t));
 if (cmd==NULL) return SWD_ERROR_OUTOFMEM;
 ack=&cmd->ack;
 cmd->bits=SWD_ACK_BITLEN;
 cmd->cmdtype=SWD_CMDTYPE_MISO_ACK;
 return swd_cmd_queue_append(swdctx->cmdq, cmd); //should be 1 on success
}

/* Append n-octet size control sequence to the context command queue. */
int swd_cmd_queue_append_mosi_control(swd_ctx_t *swdctx, char **ctlmsg, int len){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 if (ctlmsg==NULL) return SWD_ERROR_NULLPOINTER;
 if (len<=0) return SWD_ERROR_PARAM;
 int elm, res, cmdcnt=0;
 swd_cmd_t **cmd;
 cmd=calloc(len,sizeof(swd_cmd_t));
 if (cmd==NULL) return SWD_ERROR_OUTOFMEM;
 for (elm=0;elm<len;elm++){
  cmd[elm]->control=ctlmsg[elm];
  cmd[elm]->cmdtype=SWD_CMDTYPE_MOSI_CONTROL;
  cmd[elm]->bits=sizeof(ctlmsg[elm]);
  res=swd_cmd_queue_append(swdctx->cmdq, cmd[elm]); 
  if (res<=1) return ret;
  cmdcnt=+res;
 }
 return cmdcnt;
}

int swd_cmd_queue_append_swdpreset(swd_ctx_t *swdctx){
 return swd_cmd_append_control(swdctx, (char *)SWD_CMD_SWDPRESET, sizeof(SWD_CMD_SWDPRESET));
}

int swd_cmd_queue_append_jtag2swd(swd_ctx_t *swdctx){
 return swd_cmd_append_control(swdctx, (char *)SWD_CMD_JTAG2SWD, sizeof(SWD_CMD_JTAG2SWD));
}

int swd_cmd_queue_append_swd2jtag(swd_ctx_t *swdctx){
 return swd_cmd_append_control(swdctx, (char *)SWD_CMD_SWD2JTAG, sizeof(SWD_CMD_SWD2JTAG));
}

/* Check/set the bus direction to MOSI. */
int swd_bus_setdir_mosi(swd_ctx_t *swdctx){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 int res, cmdcnt=0;
 if ( swdctx->cmdq->prev==NULL || (swdctx->cmdq->cmdtype*SWD_CMDTYPE_MOSI<0) ) {
  res=swd_cmd_append_mosi_trn(swdctx);
  if (res<1) return res;
  cmdcnt=+res;
 }
 return cmdcnt;
}
 
/* Check/set the bus direction to MISO. */
int swd_bus_setdir_miso(swd_ctx_t *swdctx){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 int res, cmdcnt=0;
 if ( swdctx->cmdq->prev==NULL || (swdctx->cmdq->cmdtype*SWD_CMDTYPE_MISO<0) ) {
  res=swd_cmd_append_miso_trn(swdctx);
  if (res<0) return res;
  cmdcnt=+res;
 }
 return cmdcnt;
}

int swd_bit8_gen_request(swd_ctx_t *swdctx, char *APnDP, char *RnW, char *addr, char *request){
 /* Verify function parameters.*/
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 if (*APnDP!=0 || *APnDP!=1) return SWD_ERROR_APnDP;
 if (*RnW!=0 || *RnW!=1) return SWD_ERROR_RnW;
 if (*addr<SWD_ADDR_MINVAL || *addr>SWD_ADDR_MAXVAL) return SWD_ERROR_ADDR;

 /* Build request header content. */
 unsigned char reqhdr=0;
 int res, cmdcnt=0;
 reqhdr|=(((addr&(1<<2))?1:0)<<SWD_REQ_A2_BITNUM);
 reqhdr|=(((addr&(1<<3))?1:0)<<SWD_REQ_A3_BITNUM);
 reqhdr|=((APnDP?1:0)<<SWD_REQ_APnDP_BITNUM);
 reqhdr|=(((RnW?1:0)<<SWD_REQ_RnW_BITNUM));
 res=swd_bin8_parity_even(reqhdr);
 if (res<0 || res>1) return SWD_ERROR_PARITY;
 reqhdr|=(res<<SWD_REQ_PARITY_BITNUM);
 reqhdr|=(SWD_REQ_START_VAL<<SWD_REQ_START_BITNUM);
 reqhdr|=(SWD_REQ_STOP_VAL<<SWD_REQ_STOP_BITNUM);
 reqhdr|=(SWD_REQ_PARK_VAL<<SWD_REQ_PARK_BITNUM);

 *request=reqhdr;
 return SWD_STATUS_OK;
}


int swd_request_queue(swd_ctx_t *swdctx, char *APnDP, char *RnW, char *addr){
 /* Verify function parameters.*/
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 if (APnDP!=0 || APnDP!=1) return SWD_ERROR_APnDP;
 if (RnW!=0 || RnW!=1) return SWD_ERROR_RnW;
 if (addr<SWD_ADDR_MINVAL || addr>SWD_ADDR_MAXVAL) return SWD_ERROR_ADDR;

 int res, cmdcnt=0;
 char request;

 /* Generate request bitstream. */ 
 res=swd_bit8_gen_request(swdctx, APnDP, RnW, addr, &request);
 if (res<0) return res;

 /* Bus direction must be MOSI. */
 res=swd_bus_setdir_mosi(swdctx);
 if (res<0) return res;
 cmdcnt=+res;

 /* Append request command to the queue. */
 res=swd_cmd_append_request(swdctx, &request);
 if (res<0) return res;
 cmdcnt+=res;

 return cmdcnt; 
}

/* Perform ACK read into *ack, verify received data.
 * Return number of processed commands on the queue. */
int swd_ack_queue(swd_ctx_t *swdctx, char *ack){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 if (ack==NULL) return SWD_ERROR_NULLPOINTER;

 if (swdctx->cmdq->prev!=SWD_CMDTYPE_MOSI_REQUEST
  && swdctx->cmdq->prev!=SWD_CMDTYPE_MISO_TRN)  return SWD_ERROR_ACKORDER;

 int res, qcmdcnt=0, tcmdcnt=0;

 /* Bus direction must be MISO. */ 
 res=swd_bus_setdir_miso(swdctx);
 if (res<0) return res;
 qcmdcnt=+res;

 res=swd_cmd_append_miso_ack(swdctx, ack);
 if (res<0) return res;
 qcmdcnt=+res;

 return qcmdcnt;
}


int swd_ack_get(swd_ctx_t *swdctx, char *ack){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 if (ack==NULL) return SWD_ERROR_NULLPOINTER;
 int res, qcmdcnt=0, tcmdcnt=0;

 /* Append ACK command to the end of the queue. */
 qcmdcnt=swd_ack_queue(swdctx, ack);
 if (qcmdcnt<1) return qcmdcnt;

 /* Execute all commands on queue to get response. */
 res=swd_transfer_all(swdctx);
 if (res<0) return res;
 tcmdcnt=+res;

 /* Find last executed ACK command on the queue (ack of our interest). */
 while (cmdctx->cmdq->cmdtype!=SWD_CMDTYPE_MISO_ACK){
  if (swdctx->cmdq->prev==NULL) return SWD_ERROR_ACKMISSING;
  cmdctx->cmdq=cmdctx->cmdq->prev; 
 }
 /* If command was found and executed, return ACK code, or error code. */
 if (cmdctx->cmdq->cmdtype==SWD_CMDTYPE_MISO_ACK && cmdctx->cmdq->done){
  /* Verify value of cmd found on the queue, with pointer selected before run.*/
  if (cmdctx->cmdq->ack!=*cmd) return SWD_ERROR_ACKINVALID;
  return qcmdcnt+tcmdcnt; 
 } else return SWD_ERROR_ACK_READ;
}


int swd_data_send(swd_ctx_t *swdctx, int *data){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 if (data==NULL) return SWD_ERROR_NULLPOINTER;
 int res, parity, cmdcnt=0;

 res=swd_bus_setdir_mosi(swdctx);
 if (res<0) return ret;
 cmdcnt=+res;

 res=swd_cmd_queue_append_mosi_data_ap(swdctx, data);
 if (res<0) return ret;
 cmdcnt=+res;

 return cmdcnt;
}

int swd_data_get(swd_ctx_t *swdctx, int *data){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT;
 if (data==NULL) return SWD_ERROR_NULLPOINTER;
 int res, parity_got, parity_calc, cmdcnt=0;

 res=swd_transmit(swdctx);
 if (res<0){
  fputs("SWD: Transmit error before read operation...\n", stderr);
  return res;
 }

 res=swd_bus_setdir_miso(swdctx);
 if (res<0) return ret;
 cmdcnt=+res;

 res=swd_cmd_append_miso_data(swdctx, data);
 if (res<0) return ret;
 cmdcnt=+res;

 res=swd_cmd_queue_append_miso_parity(swdctx, &parity);
 if (res<0) return res;
 cmdcnt=+res;

 res=swd_transmit(swdctx);
 if (res<0){
  fputs("SWD: Transmit error on read operation...\n", stderr);
  return res;
 }

 parity_calc=swd_bit32_parity_even(&data);
 if (parity_got!=parity_calc) return SWD_ERROR_PARITY;
 return cmdcnt;
}




int swd_idcode_get(swd_ctx_t *swdctx, int *idcode){
 if (swdctx==NULL) return SWD_ERROR_NULLCONTEXT; 

 int res, cmdcnt=0, idcode;
 char APnDP, RnW, addr, request, ack;

 APnDP=0;
 RnW=0;
 addr=0;

 /* Request bitmask. */
res=swd_request_queue(swdctx, &APnDP, &RnW, &addr);
if (res<1) return res;
cmdcnt=+res;
res=swd_ack_get(swdctx, &ack);
if (res<1) return res;
cmdcnt=+res;
res=swd_data_read(swdctx, &idcode, &parity);
if (res<1) return res;
cmdcnt=+res;

return cmdcnt;
}

